pltS <- function(idx=c(1,7),yl=NULL,xlim=NULL,
                  col=NULL,sd=F,a=1.5,
                  lty=NULL,lgd=F,txt=NULL,cex=0.7,lwd=NULL,
                  plt="p", scl=F)
  {
    ## plt.p = plot pdf
    ## plt.S = plot survial
    if (plt=="P") plt <- "p"
    if (plt=="H") plt <- "h"
    if (plt=="s") plt <- "S"
    if (!is.element(plt,c("p","h","S"))){
      cat("\n Need to choose plt='p','h' or 'S'.\n")
      return(-1)
    }
    if (!file.exists("py-x.mdp")){ ## reading does not work
      cat("\n *** Error: could not find 'py-x.mdp.' \n",
          " *** Need to call 'ppmx(.)' first.\n")
      return(-1)
    }
    if (!file.exists("py-y.mdp")){ ## reading does not work
      cat("\n *** Error: could not find 'py-y.mdp.' \n",
          " *** Need to call 'ppmx(.)' first.\n")
      return(-1)
    }

    x <- as.matrix( read.table("py-x.mdp") )
    ## build labels for default legend
    txt2 <- "COVARIATES "
    for(i in 1:nrow(x))
      txt2 <- c(txt2,
                paste( format(x[i,],digits=2,width=4), collapse=" "))
    if (is.null(lty))  # and other parameters for pretty plots
      lty <- 1:length(idx)
    if (is.null(col))
      col <- 1:length(idx)
    if (is.null(lwd))
      lwd <- rep(2,length(idx))

    ## read in grid for y and list of covariates 
    y <- scan("py-y.mdp")
    if (is.null(xlim))
      xlim <- range(y)
    ny <- length(y)
    x <- as.matrix( read.table("py-x.mdp") )
    nx <- nrow(x)
    px <- ncol(x)
    if (max(idx)>nx){
      cat("\n *** Error: idx > nx. nx=",nx,"\n")
      return(-1);
    }

    ## now plot it
    if(plt=="p"){        # plot pdf
      S <- matrix(scan("py-p.mdp"),byrow=T,ncol=ny) # pdf
      ylab <- "PDF"
    } else if (plt=="h"){ # plot hazard
      pdf <- matrix(scan("py-p.mdp"),byrow=T,ncol=ny) # pdf
      S <- matrix(scan("py-S.mdp"),byrow=T,ncol=ny)
      S <- pdf/S
      ylab <- "HAZARD"
    } else {         # plot survival 
      S <- matrix(scan("py-S.mdp"),byrow=T,ncol=ny)
      if (scl)       # scale to S[,1]==1
        S <- S/S[,1]
      ylab <- "SURVIVAL"
    }
    if (is.null(yl))
      yl <- c(0,max(S[idx,]))
    S0 <- seq(yl[1],yl[2],length=ny) # dummy to prepare plot
    plot(y,S0,type="n",xlab="Y",ylab=ylab,bty="l",xlim=xlim,
         ylim=yl)
    matlines(y,t(S[idx,]),type="l",lwd=lwd,col=col,lty=lty)

    ## add grey shades for pointwise posterior sd
    if(sd){
      if (plt=="p")
        Si0 <- matrix(scan("py-pi.mdp"),nrow=ny)
      else 
        Si0 <- matrix(scan("py-Si.mdp"),nrow=ny)
      if (plt=="h"){
        pi0 <- matrix(scan("py-pi.mdp"),nrow=ny)
        Si0 <- pi0/Si0
      }
      nxM <- ncol(Si0)
      M <- nxM/nx
      Si <- array(Si0,dimnames=c("T","X","IT"),
                  dim=c(ny,nx,M))
      sS <- sqrt( apply(Si,c(1,2),var) )
      sS <- t(sS)  # now nx rows, ny columns, like S
      grey <- gray(9/10)
      for(i in idx){
        polyg.f <- c((S+a*sS)[i,],(S-a*sS)[i,ny:1],(S+a*sS)[i,1])
        polyg.x <- c(y[1:ny],y[ny:1],y[1])
        polygon(polyg.x,polyg.f,col=grey,border=NA)
      }
      matlines(y,t(S[idx,]),type="l",lwd=lwd,col=col,lty=lty)
    }
    ## maybe add legend
    if(lgd){
      xx <- y[ny]
      yy <- max(S)
      legend(xx,yy,legend=txt2[c(1,idx+1)],lty=c(0,lty),lwd=c(0,lwd),
             col=c(0,col),bty="n",xjust=1,yjust=1,
             cex=cex)
    }
  }
