\name{ppmx}
\alias{ppmx}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{PPMX -- Random partition with covariates}
\description{
  Fits the PPMx model for a random partition including a regression on covariates.
}
\usage{
ppmx <-  function(Yin=NULL,p0=1,p1=NULL,p2=NULL,q2=NULL,p4=0,
     censoring=0,   
     cov.Vj1=NULL, cov.mean1=NULL, cov.B1=NULL,
     cov.std1=1, cov.pi2=NULL, cov.a4=NULL,  cov.b4=NULL,
     n.iter=2000, n.discard=1000, n.reinit=10000,
     n.batch=50, m.prior=0, B.prior=0, S.prior=0, verbose=1,
     py=1, iy=0, ny=50, ygrid=NULL, xgrid=NULL,
     s=15, S.init=NULL, qq=5, R=NULL, B.scale=100, cc=5, m.init=NULL,
     a=NULL, A.inv=NULL, alpha=1, a0=1, b0=1, k0=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{Yin}{response data as (n x p0) matrix of p0-dimensional responses
             for n experimental units. Either the data matrix itself or
             a file name for a text file with the data.
	     If censoring=1 then Yin is a (n x 3) matrix with columns
             equal to the event time, an indicator for an observed event
             and the censoring time.}
  \item{p0}{dimension of the response vector for each data record }
  \item{p1}{number of continuous covariates (can be 0 for none)}
  \item{p2}{number of categorical covariates.}
  \item{q2}{vector of length p2; number of levels for each categorical covariate.}
  \item{p4}{number of count covariates.}
  \item{censoring}{indicator for censoring, censoring=1 is only allowed
    with p0=1.}
  \item{cov.Vj1}{vector of length p1, variance parameters for the
    similarity function for continuous covariates.}
  \item{mean1}{vector of length p1, prior mean of the location parameter for the
    similarity function for continuous covariates.}
  \item{mean1}{vector of length p1, prior variance for the location
    parameter in the similarity function for continuous covariates.}
  \item{cov.std1}{indicator for standardizing continuous covariates to
    sample mean 0 and standard deviation 1.}
  \item{cov.pi2}{array with p2 rows and q2[j] columns in row j.
    The j-th row defines the Dirichlet parameters for the similarity
    function for the j-th categorical covariate.}
  \item{cov.a4}{vector of length p4, defines the shape parameter of the
    Gamma prior in the similarity function for count covariates.}
  \item{cov.b4}{vector of length p4, defines the scale parameter of the
    Gamma prior in the similarity function for count covariates.}
  \item{n.iter}{number MCMC iterations }
  \item{n.discard}{initial transient}
  \item{n.reinit}{reinitialize every \code{n.reinit} iterations
    (not normally used) }
  \item{n.batch}{save imputed parameters every \code{n.batch} iterations}
  \item{n.predupdate}{save posterior predictive summaries every
    \code{n.predupdate} iterations }
  \item{m.prior}{indicator for resampling (1) versus
    fixing (0) the mean of the base measure, m}
  \item{B.prior}{indicator for resampling (1) versus
    fixing (0) the variance-covariance matrix of the base measure, B}
  \item{S.prior}{indicator for resampling (1) versus
    fixing (0) the sampling variance-covariance matrix.}
  \item{verbose}{level of comments}
  \item{py}{indicator for evaluating posterior predictive
    \eqn{p(y \mid data)} for a future observation. }
  \item{iy}{The coordinate of the response for posterior predictive
    inference. }
  \item{ny}{Size of the grid on which to evaluate posterior predictive
    inference.}
  \item{ygrid}{vector of length 2, lower and upper bound of the grid
    on which to evaluate posterior predictive inference.}
  \item{nx}{number of covariate combinations for which to carry out the
    posterior predictive inference.}
  \item{xgrid}{ \eqn{(nx \times p1+p2+p4)} matrix of covariate values 
    for which to carry out the posterior predictive inference.}
  \item{s}{degrees of freedom for the inverse Wishart prior on \eqn{S}}
  \item{S.init}{initial value for the kernel covariance matrix \eqn{S}}
  \item{qq}{degrees of freedom for the inverse Wishart
    base measure for \eqn{G_0(V_i)}}
  \item{R}{expectation of the inverse Wishart
    base measure \eqn{G_0(V_i)}}
  \item{R}{ ~~Describe \code{R} here~~ }
  \item{B.scale}{The initial value for the
    covariance matrix \eqn{B} of the base measure \eqn{G_0(\mu)=N(m,B)}
    is set to \eqn{B.scale \cdot S.init}.}
  \item{cc}{degrees of freedom of the inverse Wishart
    hyperprior for \eqn{B}}
  \item{m.init}{initial value for the
    mean \eqn{m} of the base measure \eqn{G_0(\mu)=N(m,B)}}
  \item{a}{hyperprior mean for \eqn{m}}
  \item{A.inv}{inverse of the hyperprior covariance matrix for \eqn{m}}
  \item{alpha}{initial value for the
    total mass parameter \eqn{alpha}}
  \item{a0}{hyperprior parameters for prior on total mass paramter \eqn{\alpha}}
  \item{b0}{hyperprior parameters for prior on total mass paramter \eqn{\alpha}}
  \item{k0}{initial number of distinct clusters}
}
\details{
  See \code{\link{ppmx-package}} for a statement of the probability
  model.  The function \code{ppmx} initializes and carries out MCMC
  posterior simulation.  Simulation output is saved in the working
  directory.  Change it by using \code{setwd} if desired.
  
  The parameters \code{cov.Vj1, cov.mean1, cov.B1, cov.pi2, k0,
    S.ini, R, m.init, A.inv, a, iy, ygrid, xgrid} are set to default
    values if set to NULL.
    If NULL, the arguments \code{cov.Vj1, cov.mean1, cov.B1, cov.pi2, 
    S.ini, R, m.init, A.inv} are replaced by suitable empirical moments
    of the data. The initial number of clustes \code{k0} is set to
    \code{n} if there are no categorical covariates.
    If there are categorical covariates, then \code{k0} is determined by
    the cross-tabulation of these categorical covariates.
    If \eqn{py=1} and \code{xgrid} is NULL, then \code{xgrid} is
    replaced by a default choice depending on \code{p1+p2}.
    If \eqn{p1+p2=0} then \code{xgrid} is simply chosen as the
    covariates of the first 5 data points. If \eqn{p1+p2>0} then
    \code{xgrid} is defined by all possible cross tabulations of
    the \code{p2} categorical covariates and low, medium and high values
    of the \code{p1} continuous covariates. See the file py-x.mdp for a
    copy of \code{xgrid}.
    Finally, if NULL then \code{ygrid} is set to the range of the
    response variable.
   
  Output files:
  \tabular{ll}{
  par.mdp \tab imputed hyperparameters for each iteration of the MCMC.\cr
  \tab Columns are: iteration number, \eqn{\alpha}, \eqn{\eta}, \eqn{k}\cr
  mu.mdp  \tab imputed \eqn{\mu^\star_j,~ j=1,\ldots,k}. Columns are
          iteration number, \eqn{n_j}, \eqn{\mu^\star_j}.\cr
          \tab (newlines are not alligned with j;
	       \eqn{n_j=} size of j-th cluster)\cr
  V.mdp   \tab imputed \eqn{V^\star_j}. Columns are it no, \eqn{n_j},
               \eqn{V_j} (stacked as one row vector).\cr
  member.mdp \tab imputed cluster membership indicators
            \eqn{s_i,~ i=1,\ldots,n;~ s_i\in\{0,\ldots,k-1\}}\cr
	    \tab Columns are iteration no, n clusters \eqn{k},
	    \eqn{s_i,~ i=1,\ldots,n} \cr
	    S.mdp     \tab
\eqn{(nx \times ny)}; estimated survival function \eqn{S(y_{iy} \mid
  x)}\cr
            \tab  one row per line of \code{xgrid}. \cr
	    pdf.mdp     \tab
\eqn{(nx \times ny)}; estimated pdf \eqn{p(y_{iy} \mid x)}\cr
  }
  Additional files are
  B.mdp (imputed variance-covariance matrix for
  cluster specific means), mean.mdp (imputed $m$), mj1.mdp (parameters
  for similarity function for continuous covariates), pij2.mdp (for
  categorical covariates), 
  aj4.mdp (parameters for similarity function for count covariates),
  See the example in Mueller, Quintana and Rosner (2008, Section 6.1)
  for a definition of \eqn{\mu,V,m,B} and other parameters.
  See Section 4 for a definition of the similarity functions.
  The underlying PPM is the PPM induced by a Dirichlet process prior
  with total mass parmaeter \eqn{\alpha}. The parameter \eqn{\eta} is a
  latent variable used in the MCMC transition probability for
  \eqn{\alpha}. 
 }
\value{
  The function returns no value.
  Simulation output is written to files. 
}
\references{
  the package uses the parametrization defined in:
  
  
  M\"uller, P., Quintana, F, and Rosner, G. (2008),
  ``Bayesian Clustering with Regression'', Technical report
  \url{http://odin.mdacc.tmc.edu/~pm/}
}
\author{Peter Mueller \email{pm@wotan.mdacc.tmc.edu}}
\note{Careful, \code{ppmx} writes temporary files
  into the current working directory. The same files
  are used by \code{Splt}
  to plot posterior predictive distributions.
}
\seealso{See also the \code{\link{DPpackage}}, at

  \url{http://student.kuleuven.be/~s0166452/software.html}.}
\examples{
\dontrun{
require("ppmx")

##################################################################
# prepare
##################################################################

## prepare the data file:
data(dtasim)
N <- nrow(dtasim)
## now select a sub-sample of n=200 data points for the example
n <- 200                             # desired sample size
idx <- sample(1:N,n)
Yin <- as.matrix( dtasim[idx,] )

## parameters for posterior predictive inference
xgrid <- read.table("py-x.mdp")      # cov combinations for prediction
nx <- nrow(xgrid)                    # n of covariate combinations
ny <- 50                             # size of grid for prediction

##################################################################
# call ppmx
##################################################################

ppmx(Yin,n.discard=500,n.iter=3000,
     p0=1,                           # 1-dim response
     p1=1,p2=2,p4=0,                 # 1 cont and 2 categorical cov's
     B.prior=1,S.prior=1,n.batch=50,
     xgrid=xgrid)                    # request posterior pred inference

##################################################################
# some inference summaries
##################################################################

## read in some of the inference summaries
S <-   matrix(scan("py-S.mdp"),byrow=T,ncol=ny) # S is a (nx x ny) matrix
pdf <- matrix(scan("py-p.mdp"),byrow=T,ncol=ny) # pdf    (nx x ny) matrix
ygrid <- scan("py-y.mdp")                       # grid for predictive
nx <- nrow(xgrid)

matplot(ygrid,t(S),type="l",xlab="Y",ylab="SURVIVAL BY COVARIATE",
        bty="l")
legend(x=80,y=max(S),col=0:nx,lty=0:nx,
       legend=
       c("COVARIATES",paste(xgrid[,1],xgrid[,2],xgrid[,3],sep="\t")),
       bty="n",cex=0.5)

matplot(ygrid,t(pdf),type="l",xlab="Y",ylab="DIST BY COVARIATE",
        bty="l")
legend(x=80,y=max(pdf),col=0:nx,lty=0:nx,
       legend=
       c("COVARIATES",paste(xgrid[,1],xgrid[,2],xgrid[,3],sep="\t")),
       bty="n",cex=0.5)

## same summaries using pltS(.) function
pltS(idx=1:12,plt="p",lgd=T)          # plot post pred pdf
pltS(idx=1:12,plt="h")                # ... hazard
pltS(idx=1:12,plt="S",lgd=T)          # ... survival
pltS(idx=1:2,sd=T,plt="S",lgd=T)      # with pointwise post pred sd

## some summaries of the random clustering
## report on clustering of the first n0=50 data points
n0 <- 50
y <- Yin[1:n0,1]
X <- Yin[1:n0,2:4]                  # covariates
s <- read.table("member.mdp")[,2+(1:n0)]
     ## drop first two columns (it n, n clusters) and select first n0
M <- nrow(s)                        # n saved MC imputations
## empirical frquencies of pair-wise co-clustering
idx <- order(y)
sij <- matrix(0,nrow=n0,ncol=n0)
for(i in 1:(n0-1))
  for(j in (i+1):n0)
    sij[i,j] <- sum(s[,idx[i]]==s[,idx[j]])/M
for(i in 1:n0) sij[i,i] <- 1
image(sij,col=grey((1:12)/12))

## now try after ordering by covariates
idx <- order(X[,2],X[,3],X[,1])
sij <- matrix(0,nrow=n0,ncol=n0)
for(i in 1:(n0-1))
  for(j in (i+1):n0)
    sij[i,j] <- sum(s[,idx[i]]==s[,idx[j]])/M
for(i in 1:n0) sij[i,i] <- 1
image(sij,col=grey((1:12)/12))

## track cluster membership by iteration
image(1:n0,1:M,as.matrix(t(s)), col=rainbow(12),
      xlab="OBSERV",ylab="ITERATION")

}
}