/* ===================================================================== 
   Predictives and cond expectations in MDP

   to be used with mdp.c in the same directory
/* ===================================================================== */ 
#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#include "rand.h"
#include "nrutil.h"
#include "matrix.h"
#include "vector.h"
#include "rand-nr.h" 
#include "mess.h"
#include "interface.h"
#include "ppmx.h"
#include "predict.h"

/* ***********************************************************
   data structures
/* *********************************************************** */

/* -----------------------------------------------------------
   grid for univariate pdf and exp value
/* ----------------------------------------------------------- */
struct Y_GRID{
  int ny;              /* grid size for response */
  int nx;              /* number of covariate vectors */
  int yi;              /* index of response (probably 0) */

  int n_update;       /* number of times z-value has been upd */
  double **x;         /* nx x (p-p0) matrix of covariates */
  double *y;          /* (ny x 1) grid over y */
  double **p,         /* (ny x 1) pdf on grid over y */
    **S,              /* 1-cdf */
    /* aux for updating p */
    **pi,             /* aux of unstandardized prob's */
    **Si;
};

struct Y_GRID *yalloc(void)
/* allocates memory for an X_GRID */
{
  return (struct Y_GRID *) malloc(sizeof(struct Y_GRID));
}

int p,p0,px;
/* global variables -- initialized in py_init */                

/* ***********************************************************
   univariate predictive
/* *********************************************************** */
/* -----------------------------------------------------------
   initialize univ predicitve
/* ----------------------------------------------------------- */
struct Y_GRID *py_init(struct MDP_PARS mdp, struct DTA dta)
{
  int nx, ny, yi;
  double yrange[2];
  struct Y_GRID *py;


  p = dta.p;
  p0 = dta.p0;
  px = p-p0;

  /* read pars */
  openIn("py-init.mdp");
  scanInt(" ny ", &ny);  /* grid size for response */
  scanInt(" nx ", &nx);  /* number of cov vectors */
  scanInt(" yi ", &yi);  /* index of response */

  /* alloc mem */
  py = yalloc();
  py->y = dvector(0,ny-1);
  py->x = dmatrix(0,nx-1,0,px-1);
  py->p = dmatrix(0,nx-1,0,ny-1);
  py->S = dmatrix(0,nx-1,0,ny-1);
  py->pi = dmatrix(0,nx-1,0,ny-1);
  py->Si = dmatrix(0,nx-1,0,ny-1);

  /* init members */
  py->ny=ny;
  py->nx=nx;
  py->yi=yi;
  py->n_update = 0;

  /* read y grid */
  scanDoubleArray(" ygrid ", yrange,2); /* range of y grid */
  grid(yrange[0],yrange[1],ny,py->y);

  /* read x grid */
  scanDoubleMatrix(" xgrid ", py->x, nx, px);
  return py;
}

/* -----------------------------------------------------------
   print_py
/* ----------------------------------------------------------- */
/*   print E(z|xy) */
void print_py(struct Y_GRID *py, struct MDP_PARS mdp)
{
  static int first=1;

  if (first){
    /* print y-grid */
    openOut("py-y.mdp");
    writeDoubleArray(py->y,py->ny,1);
    closeOut();
    /* print x-grid */
    openOut("py-x.mdp");
    writeDoubleMatrix(py->x,py->nx,px);
    closeOut();
    
  }

  /* write p-grid */
  openOut("py-p.mdp");
  writeDoubleMatrix(py->p,py->nx,py->ny);
  closeOut();
  openOut("py-S.mdp");
  writeDoubleMatrix(py->S,py->nx,py->ny);
  closeOut();
  if (first) 
    openOut("py-pi.mdp");
  else
    openAppend("py-pi.mdp");
  writeDoubleMatrix(py->pi,py->nx,py->ny);
  closeOut();
  if (first) 
    openOut("py-Si.mdp");
  else
    openAppend("py-Si.mdp");
  writeDoubleMatrix(py->Si,py->nx,py->ny);
  closeOut();
  
  first=0;
  
}

/* -----------------------------------------------------------
   update
/* ----------------------------------------------------------- */
/*   get expected values over grid xgrid, ygrid (global??) */
     void py_update(int time, struct Y_GRID *py, 
		    struct MDP_PARS mdp, struct DTA dta,
		    struct SIM_PAR sim)
{
  int i,j,k,
    yi,nx,ny,K,n,
    n_updates;
  double kd,alpha,an,lpr,
    sxx,sx,m,
    wmx,s,
    *yline,
    y,r,*w, *c,
    mxh, hC, lgr;


  /* set aux pars */  
  yi = py->yi;
  ny = py->ny;
  nx = py->nx;
  alpha = mdp.alpha;
  kd = 1.0*mdp.n_class;
  an = mdp.alpha+1.0*dta.n_obs;
  n = dta.n_obs;
  K = mdp.n_class;
  n_updates = py->n_update;

  /* alloc mem */
  w = dvector(0,K);
  c = dvector(0,K);
  yline= dvector(0,p);
  x_zero(yline,p);

  /* make weights */
  for (i=0;i<nx;i++){
    for(j=0;j<px;j++) /* fill covariates into fake data record */
      yline[p0+j]=py->x[i][j];
    c[K] = logpr(yline,i,K);
    w[K]=  log(alpha/an);
    for(k=0;k<K;k++){
      c[k] = logpr(yline,i,k);
      w[k]=  log(mdp.count[k]/an);
      if (k==0)        
	mxh=c[k];
      else 
	if (c[k] > mxh) mxh=c[k];
    }
    if (sim.dahl==1){ /* Dahl's method */
      c[K] = log(1.0*mdp.alpha);
      w[K] = 0;
      for(k=0, hC=0.0; k<K;k++)
	hC += exp(c[k]-mxh);
      lgr = log( n/hC ); /* multiply with r to assure sum pr*q = n-1 */
      for(k=0,wmx=0;k<K;k++){
	c[k] = c[k] - mxh /*scaling*/ + lgr; /* to stdze */
	w[k] = 0;
      }
    }
    for(k=0;k<=K;k++){
      w[k] = c[k]+w[k];
      if ( (k==0) | (w[k]> wmx)) wmx=w[k];
    }
    for(s=k=0;k<=K;k++){
      w[k] = exp(w[k]-wmx);
      s+=w[k];
    }
    x_div_r(w,s,w,K+1);

  
    /* predictive if y from G0 */
    if (p0=1){
      sxx = mdp.Vt1[mdp.n_class];
      m = mdp.mean1;
    } else{
      sxx = mdp.Vt[mdp.n_class][yi][yi];
      m = mdp.mean[yi];
    }
    sx = sqrt(sxx);
    for (j = 0; j < ny; j++){
      py->pi[i][j] = w[K]*pdfnorm(py->y[j], m, sx);
      py->Si[i][j] = w[K]*(1.0-cdfnorm(py->y[j], m, sx) );
    }// for j
    
  
    /* pred if y is in one of the existing classes */
    for (k = 0; k < mdp.n_class; k++){
      if (p0>1){
	m = mdp.mu[k][yi];
	sxx = mdp.V[k][yi][yi];
      } else {
	m = mdp.mu1[k];
	sxx = mdp.V1[k];
      }
      sx = sqrt(sxx);
      for (j = 0; j < ny; j++){
	py->pi[i][j] += w[k]*pdfnorm(py->y[j], m, sx);
	py->Si[i][j] += w[k]*(1.0-cdfnorm(py->y[j], m, sx) );
      }// for j
    }
  }// for i


  /* update py->p */
  n_updates = py->n_update;
  r = n_updates/(n_updates+1.0);
  for (i = 0; i < nx; i++)
    for(j=0;j<ny;j++){
      py->p[i][j] = r*py->p[i][j] + (1-r)*py->pi[i][j];
      py->S[i][j] = r*py->S[i][j] + (1-r)*py->Si[i][j];
  }
  
  /* increment n_update */
  py->n_update = n_updates+1;

  /* free alloc mem */
  free_dvector(w,0,K);
  free_dvector(c,0,K);
  free_dvector(yline,0,p);
  return;
}



/* ===================================================================== 
   problem specific pred routines
/* ===================================================================== */ 

static struct Y_GRID *py;

/* ***********************************************************
   pred_init
/* *********************************************************** */
void pred_init(struct MDP_PARS mdp,struct DTA dta)
{
  message("pred_init");
  if (mdp.py == 1) py = py_init(mdp,dta);
}

/* ***********************************************************
   pred_update
/* *********************************************************** */
 void pred_update(int time, struct MDP_PARS mdp, struct DTA dta,
		  struct SIM_PAR sim)
{
  if (mdp.py == 1) py_update(time,py,mdp,dta,sim);
  pred_finish(mdp);
}
/* ***********************************************************
   pred_finish
/* *********************************************************** */
void pred_finish(struct MDP_PARS mdp)
{
  if (mdp.py == 1) print_py(py,mdp);
}

